# Copyright 2006, Clemson University
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met:
# 
#    1. Redistributions of source code must retain the above copyright 
# notice, this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright 
# notice, this list of conditions and the following disclaimer in the 
# documentation and/or other materials provided with the distribution.
#    3. The name of the author may not be used to endorse or promote 
# products derived from this software without specific prior written 
# permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

WLANTrafFull instproc alloc-tcp {tcptype} {
    if {$tcptype == "Reno"} {
	set tcp [new Agent/TCP/FullTcp]
    } else {
	set tcp [new Agent/TCP/FullTcp/$tcptype]
    }
    return $tcp
}

WLANTrafFull instproc setup-tcp {tcp fid} {
    # set flow ID
    $tcp set fid_ $fid
    # register done procedure for when connection is closed
    $tcp proc done {} "$self done $tcp"
}

# These need to be here so that we can attach the
# apps to a node (done in Tcl code)
WLANTrafFull instproc alloc-client-app {} {
    return [new Application/WLANTrafFull/Client]
}

WLANTrafFull instproc alloc-server-app {} {
    return [new Application/WLANTrafFull/Server]
}

WLANTrafFull instproc done {tcp} {
    $self recycle $tcp
}

WLANTrafFull instproc now {} {
    set ns [Simulator instance]
    return [format "%.6f" [$ns now]]
}

WLANTrafFull instproc set-priorities {BSifq threshold ctcp stcp size} {
#    set threshold 10240;   # 10 KB

    if {$size < $threshold} {
	set pri 1
    } else {
	set pri 2
    }
    set qlen [$BSifq get-qlen $pri]
    set avg_qlen [$BSifq get-avg-qlen $pri]
    set qlim [$BSifq get-qlim $pri]
    set target [expr 0.75 * $qlim]
    
    if {$qlen > $target || $avg_qlen > $target} {
	# move to next priority
	set pri [expr $pri + 1]
    
	if {$pri == 2} {
	    # need to do admission control for q2
	    set qlen [$BSifq get-qlen $pri]
	    set avg_qlen [$BSifq get-avg-qlen $pri]
	    set qlim [$BSifq get-qlim $pri]
	    set target [expr 0.75 * $qlim]
    
	    if {$qlen > $target || $avg_qlen > $target} {
		# move to next priority
		set pri 3
	    }
	}
    }

    $ctcp set prio_ $pri
    $stcp set prio_ $pri

#    puts stderr "Size: $size  Qlen: $qlen  AvgQlen: $avg_qlen  Lim: $qlim  Target: $target  Priority: [$ctcp set prio_]"
}
